import React from "react";
import {
  Map,
  TileLayer,
  Marker,
  Popup,
  ZoomControl,
  Polyline
} from "react-leaflet";
import "./../css/maps.css";
import {calcDistance} from "./../helpers/distance";
import {longitude, latitude, mapWidth, mapHeight, testPath} from "./../config/variables";
import {Button, Input} from "reactstrap";
import PathButtons from "./../PathButtons";

export default class Leaflet extends React.Component{
  constructor(props){
    super(props);
    this.state = {
      latitude: latitude,
      longitude: longitude,
      zoom: 15,
      path: testPath,
      distance: 0,
    }
  }

  updateState = (key, value) => {
    this.setState({[key]: value})
    this.getDistance()
  }

  addMarker = event => {
    const { lat, lng } = event.latlng
    let pathCopy = this.state.path.slice();
    pathCopy.push([lat, lng]);
    this.setState({path: pathCopy})
    this.getDistance()
  }

  updateMarker = (idx, event) => {
    let {lat, lng} = event.target.getLatLng();
    let pathCopy = this.state.path.slice();
    pathCopy[idx] = [lat, lng]
    this.setState({path: pathCopy})
    this.getDistance()
  }

  getDistance = () => {
    let dst = calcDistance(this.state.path)
    this.setState({distance: dst})
  }

  clearPath = () => {
    this.setState({path: [], distance: 0})
  }

  render() {
    const position = [this.state.latitude, this.state.longitude]
    let markers = this.props.markers
    if (markers){
      markers = this.props.markers.map((pos, idx) =>
        <Marker position={pos} key={idx}></Marker>
      )
    }else{
      markers = <div></div>
    }
    return (
      <div>
        <div className="map">
          <Map
            center={position}
            zoom={this.state.zoom}
            zoomControl={false}
            style={{width: mapWidth, height: mapHeight}}
            onClick={this.addMarker}
          >
              <TileLayer
                attribution='&amp;copy <a href="http://osm.org/copyright">OpenStreetMap</a> contributors'
                url="https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png"
              />
              <ZoomControl position="bottomright" />

              {this.state.path.map((pos, idx) =>
                <Marker
                  position={pos}
                  draggable
                  key={idx}
                  onDragend={(e) => this.updateMarker(idx, e)}
                >
                  <Popup>
                    {idx}
                  </Popup>
                </Marker>
              )}

              <Polyline color="grey" positions={this.state.path} />

              {markers}

          </Map>
        </div>

        <div className="mapInfo">
          <Button onClick={this.clearPath}>Clear path</Button>
          <Button onClick={this.getDistance}>Calculate Distance</Button>

          <p>
            Distanz: {Math.round(this.state.distance*100)/100} km
          </p>
          <PathButtons updateState={this.updateState} path={this.state.path}/>
        </div>
      </div>
    );
  }
}